library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity svga_timing is
    Port ( pixel_clock : in std_logic;
           reset : in std_logic;
           h_synch_delay : out std_logic;			-- h_synch delayed 2 clocks to line up with DAC pipeline
           v_synch_delay : out std_logic;			-- v_synch delayed 2 clocks to line up with DAC pipeline
           comp_synch : out std_logic;				-- composite synch for DAC
           blank : out std_logic;					-- composite blanking
           vga_ram_read_address : out std_logic_vector(19 downto 0));
end svga_timing;

architecture Behavioral of svga_timing is

constant SRAM_DELAY : integer := 3;

-- 640 X 480 @ 60Hz with a 25.175 MHz pixel clock
constant H_ACTIVE : integer := 640;
constant H_FRONT_PORCH : integer := 16;
--constant H_SYNCH : integer := 96;
constant H_BACK_PORCH : integer := 48;
constant H_TOTAL : integer := 800;

--constant H_ACTIVE : integer := 10;
--constant H_FRONT_PORCH : integer := 3;
--constant H_BACK_PORCH : integer := 3;
--constant H_TOTAL : integer := 20;




constant V_ACTIVE : integer := 480;
constant V_FRONT_PORCH : integer := 11;
--constant V_SYNCH : integer := 2;
constant V_BACK_PORCH : integer := 31;
constant V_TOTAL : integer := 524;

signal line_count: std_logic_vector (9 downto 0);		-- counts the display lines
signal pixel_count : std_logic_vector (10 downto 0);	-- counts the pixels in a line	
signal h_synch : std_logic;									-- horizontal synch
signal v_synch : std_logic;									-- vertical synch
signal h_synch_delay0 : std_logic;							-- h_synch delayed 1 clock
signal v_synch_delay0 : std_logic;							-- v_synch delayed 1 clock

signal h_c_synch : std_logic;			-- horizontal component of comp synch
signal v_c_synch : std_logic;			-- vertical component of comp synch
signal h_blank : std_logic;			-- horizontal blanking
signal v_blank : std_logic;			-- vertical blanking
	
signal reset_vga_ram_read_address : std_logic;	-- flag to reset the ram address during VBI
signal hold_vga_ram_read_address : std_logic;	-- flag to hold the address during HBI

signal temp: STD_LOGIC_VECTOR (19 downto 0);		

begin


	process (pixel_clock, reset)

	

	begin
-- CREATE THE HORIZONTAL LINE PIXEL COUNTER
		if reset = '1' then													-- on reset set pixel counter to 0
				pixel_count <= "00000000000";
	
	elsif pixel_clock'event and pixel_clock='1' then			-- positive clk edge
	
		if pixel_count = (H_TOTAL - 1) then							-- last pixel in the line
				pixel_count <= "00000000000";						-- reset pixel counter
		else 	
				pixel_count <= pixel_count +1;		
		end if;

	end if;

-- CREATE THE HORIZONTAL SYNCH PULSE
	if reset = '1' then													-- on reset set h_sync to 0
			h_synch <= '0';												-- remove h_synch

	elsif pixel_clock'event and pixel_clock='1' then			-- positive clk edge
	
		if pixel_count = (H_ACTIVE + H_FRONT_PORCH -1) then	-- start of h_synch
			h_synch <= '1';
		elsif pixel_count = (H_TOTAL - H_BACK_PORCH -1) then	-- end of h_synch
			h_synch <= '0';
		end if;
	end if;


-- CREATE THE VERTICAL FRAME LINE COUNTER
	if reset = '1' then													-- on reset set line counter to 0
			line_count <= "0000000000";

	elsif pixel_clock'event and pixel_clock='1' then			-- positive clk edge
	
		if ((line_count = V_TOTAL - 1) and (pixel_count = H_TOTAL - 1)) then					-- last pixel in the line
				line_count <= "0000000000";											-- last pixel in last line of frame 
		elsif pixel_count = (H_TOTAL - 1) then
					line_count <= line_count + 1;						-- increment line counter
		end if;
	
	end if;

-- CREATE THE VERTICAL SYNCH PULSE
	if reset = '1' then							
			v_synch <= '0';												-- remove v_synch

	elsif pixel_clock'event and pixel_clock='1' then			-- positive clk edge
	
		if line_count = (V_ACTIVE + V_FRONT_PORCH -1) and pixel_count = (H_TOTAL - 1) then
			v_synch <= '1'; 												-- start of v_synch
		elsif line_count = (V_TOTAL - V_BACK_PORCH - 1)	and pixel_count = (H_TOTAL - 1) then
			v_synch <= '0'; 												-- end of v_synch
		end if;
	
	end if;

-- ADD TWO PIPELINE DELAYS TO THE SYNCHs COMPENSATE FOR THE DAC PIPELINE DELAY
	if reset = '1' then							
			h_synch_delay0 <= '0';
			v_synch_delay0 <= '0';
			h_synch_delay  <= '0';
			v_synch_delay  <= '0';

	elsif pixel_clock'event and pixel_clock='1' then		-- positive clk edge
			h_synch_delay0 <= h_synch;
			v_synch_delay0 <= v_synch;
			h_synch_delay  <= h_synch_delay0;
			v_synch_delay  <= v_synch_delay0;
	
	end if; 
	
-- CREATE THE HORIZONTAL BLANKING SIGNAL
-- the "-2" is used instead of "-1" because of the extra register delay
-- for the composite blanking signal 
	if reset = '1' then												-- on reset
			h_blank <= '0';											-- remove the h_blank

	elsif pixel_clock'event and pixel_clock='1' then		-- positive clk edge

		if (pixel_count = (H_ACTIVE -2)) then				-- start of HBI
				h_blank <= '1';
		elsif (pixel_count = (H_TOTAL -2)) then				-- end of HBI
				h_blank <= '0';
		end if;
	
	end if;

-- CREATE THE VERTICAL BLANKING SIGNAL
-- the "-2" is used instead of "-1"  in the horizontal factor because of the extra
-- register delay for the composite blanking signal 
	if reset = '1' then												-- on reset
			v_blank <= '0';											-- remove v_blank

	elsif pixel_clock'event and pixel_clock='1' then		-- positive clk edge
		
		if line_count = (V_ACTIVE - 1) and pixel_count = (H_TOTAL - 2) then
			v_blank <= '1'; 											-- start of VBI
		elsif line_count = (V_TOTAL - 1) and pixel_count = (H_TOTAL - 2) then
			v_blank <= '0';											-- end of VBI
		end if;
	
	end if;


-- CREATE THE COMPOSITE BANKING SIGNAL
	if reset = '1' then												-- on reset
			blank <= '0';												-- remove blank

	elsif pixel_clock'event and pixel_clock='1' then		-- positive clk edge
		if (h_blank or v_blank) = '1' then								-- blank during HBI or VBI
				blank <= '1';
		else 
			blank <= '0';												-- active video do not blank
		end if;
	
	end if;
		
-- CREATE THE HORIZONTAL COMPONENT OF COMP SYNCH
-- the "-2" is used instead of "-1" because of the extra register delay
-- for the composite synch
	if reset = '1' then												-- on reset
			h_c_synch <= '0';											-- remove h_c_synch

	elsif pixel_clock'event and pixel_clock='1' then		-- positive clk edge

		if pixel_count = (H_ACTIVE + H_FRONT_PORCH -2) then
	  		h_c_synch <= '1';											-- start of h_c_synch
		elsif (pixel_count = (H_TOTAL - H_BACK_PORCH -2)) then
			h_c_synch <= '0';											-- end of h_c_synch
		end if;
	
	end if;

-- CREATE THE VERTICAL COMPONENT OF COMP SYNCH 
	if reset = '1' then												-- on reset
			v_c_synch <= '0';											-- remove v_c_synch

	elsif pixel_clock'event and pixel_clock='1' then		-- positive clk edge
		if line_count = (V_ACTIVE + V_FRONT_PORCH - 1) and pixel_count = (H_TOTAL - 2) then
			v_c_synch <= '1';											-- start of v_c_synch
		elsif line_count = (V_TOTAL - V_BACK_PORCH - 1) and pixel_count = (H_TOTAL - 2) then
			v_c_synch <= '0';											-- end of v_c_synch
		end if;

	end if;

-- CREATE THE COMPOSITE SYNCH SIGNAL
	if reset = '1' then												-- on reset
			comp_synch <= '0';										-- remove comp_synch
	elsif pixel_clock'event and pixel_clock='1' then		
		comp_synch <= v_c_synch xor h_c_synch;
	end if;

-- CREATE THE RAM ADDRESS COUNTER 
	if reset = '1' then												-- on reset
			temp <= "00000000000000000000";

	elsif pixel_clock'event and pixel_clock='1' then		-- positive clk edge

		if reset_vga_ram_read_address = '1' then
			temp <= "00000000000000000000";
		elsif hold_vga_ram_read_address = '0' then
			temp <= temp + 1;
		end if;
	end if;
	vga_ram_read_address <= temp;

-- CREATE THE CONTROL SIGNALS FOR THE VGA RAM ADDRESS COUNTER
	if reset = '1' then												-- on reset
			reset_vga_ram_read_address <= '0';

	elsif pixel_clock'event and pixel_clock='1' then		-- positive clk edge
		if line_count = V_ACTIVE -1 and pixel_count =((H_TOTAL -1)- SRAM_DELAY) then
			reset_vga_ram_read_address <= '1';					-- reset the address counter at the end of active video		
		elsif line_count = V_TOTAL -1 and pixel_count =((H_TOTAL -1) - SRAM_DELAY) then
			reset_vga_ram_read_address <= '0'; 					-- re-enable the address counter at the start of active video
		end if;
	
	end if;

	if reset = '1' then												-- on reset
			hold_vga_ram_read_address <= '0';

	elsif pixel_clock'event and pixel_clock='1' then		-- positive clk edge
		if pixel_count = ((H_ACTIVE-1) - SRAM_DELAY) then	-- hold the address counter at the end of active video
 			hold_vga_ram_read_address <= '1';
		elsif pixel_count = ((H_TOTAL -1) - SRAM_DELAY) then
			hold_vga_ram_read_address <= '0';					-- re-enable the address counter at the start of active video
		end if;
	
	end if;

	end process;

end Behavioral;



